"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateAsync = exports.validateSync = void 0;
const ajv_1 = __importDefault(require("ajv"));
const ajv_keywords_1 = __importDefault(require("ajv-keywords"));
const clone_deep_1 = __importDefault(require("clone-deep"));
const errors_1 = require("./errors");
const functions_1 = require("./util/functions");
const ajv_custom_1 = __importDefault(require("./util/ajv-custom"));
const compiler = new ajv_1.default({
    transpile: false,
    coerceTypes: false,
    removeAdditional: 'failing',
    useDefaults: 'empty',
    strictNumbers: true,
    nullable: true,
    verbose: true,
    messages: false,
});
ajv_keywords_1.default(compiler, ['transform', 'regexp', 'instanceof']);
ajv_custom_1.default(compiler);
const cached = new WeakMap();
function wrap(shape) {
    var _a;
    const schema = functions_1.transformIntoValidationSchema(shape.schema);
    return {
        $async: functions_1.checkAsyncSchema(schema),
        type: 'object',
        properties: { value: schema },
        required: ((_a = shape.options) === null || _a === void 0 ? void 0 : _a.optional) ? [] : ['value'],
        errorMessage: {
            required: 'is required',
        },
    };
}
function compile(shape) {
    if (!cached.has(shape)) {
        cached.set(shape, compiler.compile(wrap(shape)));
    }
    const compiled = cached.get(shape);
    if (!compiled) {
        throw new Error('Cannot get compiled validate function');
    }
    return compiled;
}
/**
 * 同步验证数据是否符合「形状」
 * - 验证通过时，返回验证后的数据
 * - 验证不通过时，抛出 ValidationError 的实例
 * - 不支持异步验证
 * @param shape 要验证的「形状」
 * @param value 要验证的数据
 * @param clone 是否在验证前深拷贝数据（默认开启）
 */
function validateSync(shape, value, clone = true) {
    const validator = compile(shape);
    if (validator.$async) {
        throw new Error('Async validation is not supported, should use validateAsync instead');
    }
    const wrapped = { value };
    if (clone) {
        wrapped.value = clone_deep_1.default(wrapped.value, true);
    }
    const valid = validator(wrapped);
    if (!valid) {
        throw new errors_1.ValidationError(validator.errors);
    }
    return wrapped.value;
}
exports.validateSync = validateSync;
/**
 * 异步验证数据是否符合「形状」
 * - 验证通过时，Promise 返回验证后的数据
 * - 验证不通过时，Promise 抛出 ValidationError 的实例
 * - 支持异步验证
 * @param shape 要验证的「形状」
 * @param value 要验证的数据
 * @param clone 是否在验证前深拷贝数据（默认开启）
 */
async function validateAsync(shape, value, clone = true) {
    const validator = compile(shape);
    const wrapped = { value };
    if (clone) {
        wrapped.value = clone_deep_1.default(wrapped.value, true);
    }
    const valid = validator(wrapped);
    if (!valid) {
        if (validator.errors && validator.errors.length > 0) {
            throw new errors_1.ValidationError(validator.errors);
        }
        throw new Error('Validation failed');
    }
    else if (typeof valid !== 'boolean') {
        if (typeof valid.then !== 'function') {
            throw new TypeError('There is some trouble with ajv, the return value is neither boolean nor promise');
        }
        await valid.then(functions_1.noop, (error) => {
            if (!(error instanceof ajv_1.default.ValidationError)) {
                throw error;
            }
            throw new errors_1.ValidationError(error.errors);
        });
    }
    return wrapped.value;
}
exports.validateAsync = validateAsync;
